# coding: utf-8
# pylint: disable=no-member

from sqlalchemy import MetaData, Table, Column, UnicodeText, String

from tionix_client.os_clients.keystone_client import get_admin_keystone_client
from tionix_vdi_server.core.helpers import get_keystone_users
from tionix_vdi_server.settings import CONF


class ObjectType(object):
    Server = 'server'
    Project = 'project'

    TYPES = (
        Server,
        Project,
    )


def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)

    journal_table = Table('journal', meta, autoload=True)

    parent_request_id = Column('parent_request_id', String(64), nullable=True)
    journal_table.create_column(parent_request_id)

    object_type = Column('object_type', String(64), nullable=True)
    journal_table.create_column(object_type)

    object_id = Column('object_id', String(64), nullable=True)
    journal_table.create_column(object_id)

    status_before = Column('status_before', String(64), nullable=True)
    journal_table.create_column(status_before)

    status_after = Column('status_after', String(64), nullable=True)
    journal_table.create_column(status_after)

    user_name = Column('user_name', String(64), nullable=True)
    journal_table.create_column(user_name)

    result = Column('result', UnicodeText(), nullable=True)
    journal_table.create_column(result)

    journal_rows = journal_table.select().execute().fetchall()
    if journal_rows:
        keystone_client = get_admin_keystone_client(CONF)
        domains_ids = [domain.id
                       for domain in keystone_client.domains.list()]

        keystone_users = get_keystone_users(domains_ids)

        for journal in journal_rows:
            # pylint: disable=no-value-for-parameter
            journal_table.update().where(
                journal_table.c.id == journal.id
            ).values(object_id=journal.instance_id,
                     object_type=ObjectType.Server,
                     user_name=journal.request_user_name,
                     result=keystone_users.get(journal.user_id)).execute()

            journal_table.update().where(
                journal_table.c.id == journal.id
            ).values(user_id=journal.request_user_id).execute()

    journal_table.c.instance_id.drop()
    journal_table.c.request_user_id.drop()
    journal_table.c.request_user_name.drop()
    journal_table.c.request_body.drop()


def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)

    journal_table = Table('journal', meta, autoload=True)
    journal_table.drop_column('parent_request_id')
    journal_table.drop_column('object_type')
    journal_table.drop_column('object_id')
    journal_table.drop_column('status_before')
    journal_table.drop_column('status_after')
    journal_table.drop_column('user_name')
    journal_table.drop_column('result')

    journal_table.create_column(
        Column('instance_id', String(64), nullable=False)
    )
    journal_table.create_column(
        Column('request_user_id', String(64), nullable=False)
    )
    journal_table.create_column(
        Column('request_user_name', String(64), nullable=False)
    )
    journal_table.create_column(Column('request_body', UnicodeText()))
