# coding: utf-8

from sqlalchemy import (
    MetaData, String, Table, Column, Integer, DateTime, UnicodeText)


def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)

    requests = Table(
        'shadow_requests', meta,
        Column('created_at', DateTime),
        Column('updated_at', DateTime),
        Column('deleted_at', DateTime),
        Column('deleted', Integer),
        Column('id', Integer, primary_key=True),

        Column('username', String(64), nullable=False),
        Column('user_id', String(64), nullable=False),
        Column('project_id', String(64), nullable=False),
        Column('request_id', String(64), nullable=False),
        Column('completed_at', DateTime, nullable=True),

        Column('instance_id', String(64), nullable=True),
        Column('vm_ip', String(64), nullable=True),
        Column('task_id', String(255), nullable=True),
        Column('error', UnicodeText, nullable=True),
        Column('status', String(64), nullable=False),
        Column('result', UnicodeText(), nullable=True),
        mysql_charset='utf8'
    )
    requests.create()

    servers_table = Table(
        'shadow_servers', meta,
        Column('id', Integer, primary_key=True, nullable=False),
        Column('created_at', DateTime),
        Column('updated_at', DateTime),
        Column('deleted_at', DateTime),
        Column('deleted', Integer),
        Column('instance_id', String(36), nullable=False),
        Column('tenant_id', String(64), nullable=False),
        Column('sla_1', String(16), nullable=False),
        Column('sla_2', String(16), nullable=False),
        Column('last_session_id', String(64), nullable=True),
        Column('associated_security_group_id', String(36), nullable=True),
        Column('operation_mode', String(10)),
        mysql_charset='utf8'
    )
    servers_table.create()

    tnx_sessions = Table(
        'shadow_sessions', meta,
        Column('id', Integer, primary_key=True),
        Column('created_at', DateTime),
        Column('updated_at', DateTime),
        Column('deleted_at', DateTime),
        Column('deleted', Integer),
        Column('instance_id', String(64), nullable=False),
        Column('user_id', String(64), nullable=False),
        Column('connection', String(10)),
        Column('closed_at', DateTime),
        Column('session_id', String(64)),
        Column('client_ip', String(64)),
        Column('client_os', String(64)),
        Column('client_info', String(64)),
        Column('user_name', String(255), nullable=False),
        mysql_charset='utf8'
    )
    tnx_sessions.create()

    tnx_tenants = Table(
        'shadow_tenants', meta,
        Column('created_at', DateTime),
        Column('updated_at', DateTime),
        Column('deleted_at', DateTime),
        Column('deleted', Integer),
        Column('id', Integer, primary_key=True),
        Column('tenant_id', String(64), nullable=False),
        Column('image_id', String(36), nullable=False),
        Column('flavor_id', String(255), nullable=False),
        Column('network_id', String(36), nullable=True),
        Column('operation_mode', String(10), nullable=False),
        Column('project_type', String(64), nullable=False),
        Column('availability_zone', String(255)),
        mysql_charset='utf8'
    )
    tnx_tenants.create()

    tenant_security_groups = Table(
        'shadow_tenant_security_groups', meta,
        Column('created_at', DateTime),
        Column('updated_at', DateTime),
        Column('deleted_at', DateTime),
        Column('deleted', Integer),
        Column('id', Integer, primary_key=True),
        Column('tnx_tenant_id', String(64)),
        Column('security_group_id', String(36)),
        mysql_charset='utf8'
    )
    tenant_security_groups.create()

    tnx_users = Table(
        'shadow_users', meta,
        Column('created_at', DateTime),
        Column('updated_at', DateTime),
        Column('deleted_at', DateTime),
        Column('deleted', Integer),
        Column('id', Integer, primary_key=True),
        Column('instance_id', String(64)),
        Column('user_id', String(64)),
        mysql_charset='utf8'
    )
    tnx_users.create()

    tnx_users = Table(
        'shadow_user_groups', meta,
        Column('created_at', DateTime),
        Column('updated_at', DateTime),
        Column('deleted_at', DateTime),
        Column('deleted', Integer),
        Column('id', Integer, primary_key=True),
        Column('instance_id', String(64)),
        Column('group_id', String(64)),
        mysql_charset='utf8'
    )
    tnx_users.create()


def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    tables = ('servers', 'requests', 'sessions', 'tenants',
              'tenant_security_groups', 'users', 'user_groups', )
    for table_name in tables:
        table = Table('shadow_' + table_name, meta)
        table.drop()
